///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor: Xilinx
// \   \   \/    Version: v1.0.1
//  \   \        Filename: SlaveSerial.vhd
//  /   /        Date Last Modified:  April 16, 2009
// /___/   /\    Date Created: August 29, 2001
// \   \  /  \
//  \___\/\___\
// 
//Device: CoolRunner-II CPLD
//Purpose: CPLD design for Slave SelectMAPx8 interface to microprocessor.
//Reference:
//    XAPP502
//Revision History:
//    Rev 1.0 - First created, Mark Ng, Aug 29, 2001.
//    Rev 1.0.1 - Changed illegal reg declarations to legal wire declarations
//                for Configuration_Register_ADDR, Program_Register_ADDR, 
//                Input_Register_ADDR, and Input_Register_Data; and updated 
//                Xilinx standard Verilog header, ayang/randal, Apr 16, 2009.
/////////////////////////////////////////////////////////////////////////////////////////////
// Copyright  2001 - 2009, Xilinx, Inc.
// This file contains confidential and proprietary information of Xilinx, Inc. and is
// protected under U.S. and international copyright and other intellectual property laws.
/////////////////////////////////////////////////////////////////////////////////////////////
//
// Disclaimer:
//        This disclaimer is not a license and does not grant any rights to the materials
//        distributed herewith. Except as otherwise provided in a valid license issued to
//        you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
//        MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
//        DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
//        INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
//        OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
//        (whether in contract or tort, including negligence, or under any other theory
//        of liability) for any loss or damage of any kind or nature related to, arising
//        under or in connection with these materials, including for any direct, or any
//        indirect, special, incidental, or consequential loss or damage (including loss
//        of data, profits, goodwill, or any type of loss or damage suffered as a result
//        of any action brought by a third party) even if such damage or loss was
//        reasonably foreseeable or Xilinx had been advised of the possibility of the same.
// CRITICAL APPLICATIONS
//        Xilinx products are not designed or intended to be fail-safe, or for use in any
//        application requiring fail-safe performance, such as life-support or safety
//        devices or systems, Class III medical devices, nuclear facilities, applications
//        related to the deployment of airbags, or any other applications that could lead
//        to death, personal injury, or severe property or environmental damage
//        (individually and collectively, "Critical Applications"). Customer assumes the
//        sole risk and liability of any use of Xilinx products in Critical Applications,
//        subject only to applicable laws and regulations governing limitations on product
//        liability.
/////////////////////////////////////////////////////////////////////////////////////////////

module SelectMAP(
  ADDR,DATA,
  WEn,
  OEn,
  CS1n,
  Virtex_CCLK,
  Virtex_WRITE,
  Virtex_PROGRAM,
  Virtex_BUSY,
  Virtex_DONE,
  Virtex_INIT,
  Virtex_CS,
  Virtex_D7,
  Virtex_D6,
  Virtex_D5,
  Virtex_D4,
  Virtex_D3,
  Virtex_D2,
  Virtex_D1,
  Virtex_D0);

	//Original Spingboard Signals:

  input [23:0] ADDR;
  inout [15:0] DATA;
  input WEn;
  input OEn;
  input CS1n;

	//Virtex Configuration Signals:

  output Virtex_CCLK;
  output Virtex_WRITE;
  output Virtex_PROGRAM;
  input Virtex_BUSY;
  input Virtex_DONE;
  input Virtex_INIT;
  output Virtex_CS;
  output Virtex_D7;
  output Virtex_D6;
  output Virtex_D5;
  output Virtex_D4;
  output Virtex_D3;
  output Virtex_D2;
  output Virtex_D1;
  output Virtex_D0;

  wire [23:0] Configuration_Register_ADDR = 24'h0000;
	wire [23:0] Program_Register_ADDR = 24'h0002;
	wire [23:0] Input_Register_ADDR = 24'h0004;
	 
	wire Output_Buffer_Enable; //Output Buffer Enable Signal
	 
	reg Input_Reg_D2;
	reg Input_Reg_D1;
	reg Input_Reg_D0;

	//Internal Wire and Regs
	wire [15:0] Input_Register_Data;
	reg Virtex_D7;
  reg Virtex_D6;
  reg Virtex_D5;
  reg Virtex_D4;
  reg Virtex_D3;
  reg Virtex_D2;
  reg Virtex_D1;
  reg Virtex_D0;
  reg Virtex_CCLK;
	reg Virtex_CS;
	reg Virtex_WRITE;
	reg Virtex_PROGRAM_OE;

	
	assign Output_Buffer_Enable = !OEn & !CS1n;

	assign DATA = Output_Buffer_Enable ? Input_Register_Data : 16'hZZZZ;
  
  //Configuration Register

	always @(posedge WEn)
	begin
	  if (!CS1n)
		begin
		  if (ADDR == Configuration_Register_ADDR)
		  begin
	  	  Virtex_D7 = DATA[15];
		    Virtex_D6 = DATA[14];
			 	Virtex_D5 = DATA[13];
			  Virtex_D4 = DATA[12];
				Virtex_D3 = DATA[11];
				Virtex_D2 = DATA[10];
				Virtex_D1 = DATA[9];
				Virtex_D0 = DATA[8];
				Virtex_CCLK = DATA[0];
			end
		end
	end

  //Program Register
	
	always @(WEn)
	begin
	  if (!CS1n)
	  begin 
	    if (ADDR == Program_Register_ADDR)
 	    begin
			  Virtex_CS = DATA[2];
	 		  Virtex_WRITE = DATA[1];
				Virtex_PROGRAM_OE = DATA[0];
			end
		end
	end
			 
	assign Virtex_PROGRAM = Virtex_PROGRAM_OE ? 1'b0 : 1'bZ;

  //Input Register

	always @(negedge OEn)
	begin
	  if (CS1n) 
	  begin
	    if (ADDR == Input_Register_ADDR)
	    begin
	      Input_Reg_D2 = Virtex_BUSY;
		    Input_Reg_D1 = Virtex_DONE;
		    Input_Reg_D0 = Virtex_INIT;
	    end
    end
	end

	assign Input_Register_Data = {13'h0000, Input_Reg_D2, Input_Reg_D1, Input_Reg_D0};
	
endmodule
