///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor: Xilinx
// \   \   \/    Version: v1.0.1
//  \   \        Filename: SlaveSerial.vhd
//  /   /        Date Last Modified:  April 16, 2009
// /___/   /\    Date Created: August 29, 2001
// \   \  /  \
//  \___\/\___\
// 
//Device: CoolRunner-II CPLD
//Purpose: CPLD design for Slave Serial interface to microprocessor.
//Reference:
//    XAPP502
//Revision History:
//    Rev 1.0 - First created, Mark Ng, Aug 29, 2001.
//    Rev 1.0.1 - Changed illegal reg declarations to legal wire declarations
//                for Configuration_Register_ADDR, Program_Register_ADDR, 
//                Input_Register_ADDR, and Input_Register_Data; and updated 
//                Xilinx standard Verilog header, ayang/randal, Apr 16, 2009.
/////////////////////////////////////////////////////////////////////////////////////////////
// Copyright  2001 - 2009, Xilinx, Inc.
// This file contains confidential and proprietary information of Xilinx, Inc. and is
// protected under U.S. and international copyright and other intellectual property laws.
/////////////////////////////////////////////////////////////////////////////////////////////
//
// Disclaimer:
//        This disclaimer is not a license and does not grant any rights to the materials
//        distributed herewith. Except as otherwise provided in a valid license issued to
//        you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
//        MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
//        DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
//        INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
//        OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
//        (whether in contract or tort, including negligence, or under any other theory
//        of liability) for any loss or damage of any kind or nature related to, arising
//        under or in connection with these materials, including for any direct, or any
//        indirect, special, incidental, or consequential loss or damage (including loss
//        of data, profits, goodwill, or any type of loss or damage suffered as a result
//        of any action brought by a third party) even if such damage or loss was
//        reasonably foreseeable or Xilinx had been advised of the possibility of the same.
// CRITICAL APPLICATIONS
//        Xilinx products are not designed or intended to be fail-safe, or for use in any
//        application requiring fail-safe performance, such as life-support or safety
//        devices or systems, Class III medical devices, nuclear facilities, applications
//        related to the deployment of airbags, or any other applications that could lead
//        to death, personal injury, or severe property or environmental damage
//        (individually and collectively, "Critical Applications"). Customer assumes the
//        sole risk and liability of any use of Xilinx products in Critical Applications,
//        subject only to applicable laws and regulations governing limitations on product
//        liability.
/////////////////////////////////////////////////////////////////////////////////////////////

module SlaveSerial(
  ADDR,DATA,
  WEn,
  OEn,
  CS1n,
  Virtex_CCLK,
  Virtex_PROGRAM,
  Virtex_DONE,
  Virtex_INIT,
  Virtex_Din
  );
    
  // Original Springboard Signals:
  input [23:0] ADDR;
  inout [15:0] DATA;
  input WEn;
  input OEn;
  input CS1n;
	
  // Virtex Configuration Signals:
  output Virtex_CCLK;
  output Virtex_PROGRAM;
  input  Virtex_DONE;
  input  Virtex_INIT;
  output Virtex_Din;

 
	wire Output_Buffer_Enable; //Output Buffer Enable Signal
	
	reg Input_Reg_D1;
	reg Input_Reg_D0;

	//Internal Wire and Regs

	wire [23:0] Configuration_Register_ADDR = 24'h000000;
	wire [23:0] Program_Register_ADDR = 24'h000002;
	wire [23:0] Input_Register_ADDR = 24'h000004;

  wire [15:0] Input_Register_Data;
  reg Virtex_CCLK;
	reg Virtex_PROGRAM_OE;
	reg Virtex_Din;
	 
	assign Output_Buffer_Enable = !OEn & !CS1n;

	assign DATA = Output_Buffer_Enable ? Input_Register_Data : 16'hZZZZ;
  
  //Configuration Register

	always @(posedge WEn)
  begin
	  if (!CS1n)
    begin
	    if (ADDR == Configuration_Register_ADDR)
      begin
	      Virtex_CCLK = DATA[1];
	      Virtex_Din = DATA[0];
      end
    end
  end

  //Program Register
	
	always @(WEn)
  begin
	  if (!CS1n)
	  begin 
		  if (ADDR == Program_Register_ADDR)
		  begin
				Virtex_PROGRAM_OE = DATA[0];
		  end
	  end
	end
			 
  assign Virtex_PROGRAM = Virtex_PROGRAM_OE ? 1'b0 : 1'bZ;

  //Input Register

  always @(negedge OEn)
  begin
	if (CS1n) 
	begin
	  if (ADDR == Input_Register_ADDR)
	  begin
	    Input_Reg_D1 = Virtex_DONE;
	    Input_Reg_D0 = Virtex_INIT;
	  end
	end
  end

  assign Input_Register_Data = {14'b00000000000000, Input_Reg_D1, Input_Reg_D0};
      
endmodule
