-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: v1.0.1
--  \   \        Filename: SlaveSerial.vhd
--  /   /        Date Last Modified:  April 16, 2009
-- /___/   /\    Date Created: August 29, 2001
-- \   \  /  \
--  \___\/\___\
-- 
--Device: CoolRunner-II CPLD
--Purpose: CPLD design for Slave Serial interface to microprocessor.
--Reference:
--    XAPP502
--Revision History:
--    Rev 1.0 - First created, Mark Ng, Aug 29, 2001.
--    Rev 1.0.1 - Updated to Xilinx standard VHDL header, randal, Apr 16, 2009.
---------------------------------------------------------------------------------------------
-- Copyright  2001 - 2009, Xilinx, Inc.
-- This file contains confidential and proprietary information of Xilinx, Inc. and is
-- protected under U.S. and international copyright and other intellectual property laws.
---------------------------------------------------------------------------------------------
--
-- Disclaimer:
--        This disclaimer is not a license and does not grant any rights to the materials
--        distributed herewith. Except as otherwise provided in a valid license issued to
--        you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
--        MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
--        DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
--        INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
--        OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
--        (whether in contract or tort, including negligence, or under any other theory
--        of liability) for any loss or damage of any kind or nature related to, arising
--        under or in connection with these materials, including for any direct, or any
--        indirect, special, incidental, or consequential loss or damage (including loss
--        of data, profits, goodwill, or any type of loss or damage suffered as a result
--        of any action brought by a third party) even if such damage or loss was
--        reasonably foreseeable or Xilinx had been advised of the possibility of the same.
-- CRITICAL APPLICATIONS
--        Xilinx products are not designed or intended to be fail-safe, or for use in any
--        application requiring fail-safe performance, such as life-support or safety
--        devices or systems, Class III medical devices, nuclear facilities, applications
--        related to the deployment of airbags, or any other applications that could lead
--        to death, personal injury, or severe property or environmental damage
--        (individually and collectively, "Critical Applications"). Customer assumes the
--        sole risk and liability of any use of Xilinx products in Critical Applications,
--        subject only to applicable laws and regulations governing limitations on product
--        liability.
---------------------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity SlaveSerial is
	port(
	
	-- Original Springboard Signals:
	
	ADDR 	: in 		STD_LOGIC_VECTOR(23 DOWNTO 0);
	DATA	: inout STD_LOGIC_VECTOR(15 DOWNTO 0);
	WEn  	: in 		STD_LOGIC;
	OEn 	: in 		STD_LOGIC;
	CS1n  : in 		STD_LOGIC;

	-- Virtex Configuration Signals:

	Virtex_CCLK 	  : out STD_LOGIC;
	
	Virtex_PROGRAM	: out STD_LOGIC;
	
	Virtex_DONE 	  : in  STD_LOGIC;
	Virtex_INIT 	  : in  STD_LOGIC;
	Virtex_Din    	: out STD_LOGIC

 );


end SlaveSerial;

architecture BEHAVE of SlaveSerial is

constant Configuration_Register_ADDR	                                       
	: STD_LOGIC_VECTOR(23 downto 0) := "000000000000000000000000";    
constant Program_Register_ADDR 			 
	: STD_LOGIC_VECTOR(23 downto 0) := "000000000000000000000010";
constant Input_Register_ADDR 				
	: STD_LOGIC_VECTOR(23 downto 0) := "000000000000000000000100";

signal Output_Buffer_Enable	: std_logic;	-- Output Buffer Enable Signal

signal input_reg_d1		: std_logic;
signal input_reg_d0		: std_logic;
signal input_register_data 	: std_logic_vector(15 DOWNTO 0);

signal Virtex_PROGRAM_OE	: std_logic;

begin



Output_Buffer_Enable <= not OEn and not CS1n;

DATA(15 DOWNTO 0) <= Input_Register_Data(15 DOWNTO 0) 
		when (Output_Buffer_Enable = '1') else (others => 'Z');

--Configuration Register

process(ADDR, DATA, CS1n, WEn)
begin
	if(WEn'event and WEn='1') then
		if(CS1n = '0') then
		  if (ADDR = Configuration_Register_ADDR) then
		  	Virtex_CCLK <= DATA(1);
			  Virtex_Din  <= DATA(0);
		  end if;
		end if;
	end if;
end process;


--Program Register

process(ADDR, DATA, CS1n, WEn)
begin
	if(WEn'event and WEn='1') then
		if(CS1n = '0') then
		  if (ADDR = Program_Register_ADDR) then
			  Virtex_PROGRAM_OE <= DATA(0);
		  end if;
		end if;
	end if;
end process;

Virtex_PROGRAM <= '0' when Virtex_PROGRAM_OE = '1' else 'Z';

--Input Register

process(ADDR, DATA, CS1n, OEn)
begin

	if(CS1n = '1') then
		if(OEn'event and OEn='0') then
		  if (ADDR = Input_Register_ADDR) then
 			  Input_reg_D1 		<= Virtex_DONE;
			  Input_reg_D0		<= Virtex_INIT;
		  end if;
		end if;
	end if;
end process;

Input_Register_Data <= ("00000000000000" & Input_Reg_D1  & Input_Reg_D0);


END BEHAVE;
